From: Yinghai Lu <yinghai@kernel.org>
To: Prakash Punnoor <prakash@punnoor.de>,
Jesse Barnes <jbarnes@virtuousgeek.org>
Cc: Matthew Wilcox <matthew@wil.cx>,
Andrew Morton <akpm@linux-foundation.org>,
Ingo Molnar <mingo@elte.hu>,
"Eric W. Biederman" <ebiederm@xmission.com>,
Robert Hancock <hancockrwd@gmail.com>,
david@lang.hm, linux-kernel <linux-kernel@vger.kernel.org>,
linux-pci@vger.kernel.org
Subject: [PATCH] pci: don't enable too much HT MSI mapping -v6
Date: Sun, 29 Mar 2009 12:30:05 -0700 [thread overview]
Message-ID: <49CFCC3D.5020205@kernel.org> (raw)
In-Reply-To: <200903282316.55142.prakash@punnoor.de>
Impact: fix bug
Prakash reported that his c51-mcp51 system ondie sound card doesn't work MSI
but if he hack out the HT-MSI on mcp51, the MSI will work well with sound card.
this patch rework the nv_msi_ht_cap_quirk()
and will only try to avoid to enable ht_msi on device following that root dev,
and don't touch that root dev
v6: only do that trick with end_device on the chain.
Reported-by: Prakash Punnoor <prakash@punnoor.de>
Tested-by: Prakash Punnoor <prakash@punnoor.de>
Signed-off-by: Yinghai Lu <yinghai@kernel.org>
diff --git a/drivers/pci/quirks.c b/drivers/pci/quirks.c
index faf02dd..52714f2 100644
--- a/drivers/pci/quirks.c
+++ b/drivers/pci/quirks.c
@@ -2206,6 +2206,33 @@ static int __devinit host_bridge_with_leaf(struct pci_dev *host_bridge)
return found;
}
+#define PCI_HT_CAP_SLAVE_CTRL0 4 /* link control */
+#define PCI_HT_CAP_SLAVE_CTRL1 8 /* link control to */
+
+static int __devinit is_end_of_ht_chain(struct pci_dev *dev)
+{
+ int pos, ctrl_off;
+ int end = 0;
+ u16 flags, ctrl;
+
+ pos = pci_find_ht_capability(dev, HT_CAPTYPE_SLAVE);
+
+ if (!pos)
+ goto out;
+
+ pci_read_config_word(dev, pos + PCI_CAP_FLAGS, &flags);
+
+ ctrl_off = ((flags >> 10) & 1) ?
+ PCI_HT_CAP_SLAVE_CTRL0 : PCI_HT_CAP_SLAVE_CTRL1;
+ pci_read_config_word(dev, pos + ctrl_off, &ctrl);
+
+ if (ctrl & (1 << 6))
+ end = 1;
+
+out:
+ return end;
+}
+
static void __devinit nv_ht_enable_msi_mapping(struct pci_dev *dev)
{
struct pci_dev *host_bridge;
@@ -2230,8 +2257,9 @@ static void __devinit nv_ht_enable_msi_mapping(struct pci_dev *dev)
if (!found)
return;
- /* don't enable host_bridge with leaf directly here */
- if (host_bridge == dev && host_bridge_with_leaf(host_bridge))
+ /* don't enable end_device/host_bridge with leaf directly here */
+ if (host_bridge == dev && is_end_of_ht_chain(host_bridge) &&
+ host_bridge_with_leaf(host_bridge))
goto out;
/* root did that ! */
next prev parent reply other threads:[~2009-03-29 19:31 UTC|newest]
Thread overview: 100+ messages / expand[flat|nested] mbox.gz Atom feed top
2009-02-17 22:24 mpt fusion broken sometime since 2.6.24 david
2009-02-17 21:44 ` david
2009-02-17 23:00 ` Matthew Wilcox
2009-02-17 23:07 ` david
2009-02-17 23:11 ` Matthew Wilcox
2009-02-17 23:20 ` Yinghai Lu
2009-02-17 23:21 ` david
2009-02-17 23:11 ` Yinghai Lu
2009-02-17 23:20 ` david
2009-02-17 23:22 ` Yinghai Lu
2009-02-17 23:37 ` david
2009-02-18 2:01 ` Yinghai Lu
2009-02-18 2:03 ` david
2009-02-18 2:14 ` Yinghai Lu
2009-02-18 2:32 ` david
2009-02-18 2:28 ` Yinghai Lu
2009-02-18 2:34 ` david
2009-02-18 2:40 ` david
2009-02-18 2:49 ` Yinghai Lu
2009-02-18 3:26 ` david
2009-02-18 3:28 ` Yinghai Lu
2009-02-18 3:35 ` david
2009-02-18 3:45 ` david
2009-02-18 3:47 ` Yinghai Lu
2009-02-18 4:14 ` david
2009-02-18 4:36 ` Yinghai Lu
2009-02-18 5:06 ` Yinghai Lu
2009-02-18 5:10 ` Yinghai Lu
2009-02-18 4:40 ` [PATCH] pci: enable MSI on 8132 Yinghai Lu
2009-02-18 12:21 ` Matthew Wilcox
2009-02-18 12:27 ` david
2009-02-18 18:04 ` Andrew Morton
2009-02-18 18:18 ` david
2009-02-18 18:32 ` Greg KH
2009-02-18 18:38 ` James Bottomley
2009-02-18 18:40 ` david
2009-02-18 19:08 ` Yinghai Lu
2009-02-18 19:14 ` James Bottomley
2009-02-18 19:25 ` Yinghai Lu
2009-02-18 20:12 ` Jeff Garzik
2009-02-18 20:18 ` James Bottomley
2009-02-18 19:15 ` Andrew Morton
2009-02-18 19:29 ` Yinghai Lu
2009-02-18 19:33 ` James Bottomley
2009-02-19 4:21 ` Prakash, Sathya
2009-02-18 19:00 ` Jesse Barnes
2009-02-19 3:39 ` Robert Hancock
2009-02-21 7:50 ` Eric W. Biederman
2009-02-21 8:31 ` Yinghai Lu
2009-02-21 8:58 ` Eric W. Biederman
2009-02-21 10:23 ` Yinghai Lu
2009-02-21 18:59 ` Robert Hancock
2009-02-22 12:08 ` Prakash Punnoor
2009-02-22 12:54 ` Eric W. Biederman
2009-02-22 15:17 ` Prakash Punnoor
2009-02-22 21:45 ` Yinghai Lu
2009-02-22 22:07 ` Yinghai Lu
2009-02-23 6:18 ` Yinghai Lu
2009-02-23 18:21 ` Prakash Punnoor
2009-02-23 18:50 ` Yinghai Lu
2009-02-23 19:01 ` Prakash Punnoor
2009-02-23 19:51 ` [PATCH] pci: don't enable too many HT MSI mapping Yinghai Lu
2009-02-24 17:37 ` Jesse Barnes
2009-02-27 6:52 ` Prakash Punnoor
2009-02-27 20:59 ` Yinghai Lu
2009-02-28 8:25 ` Prakash Punnoor
2009-02-28 20:57 ` Yinghai Lu
2009-02-28 22:43 ` Yinghai Lu
2009-03-01 7:50 ` Prakash Punnoor
2009-03-01 7:58 ` Prakash Punnoor
2009-03-01 8:12 ` Yinghai Lu
2009-03-01 8:29 ` Prakash Punnoor
2009-03-04 7:15 ` Prakash Punnoor
2009-03-04 8:21 ` Yinghai Lu
2009-03-05 17:01 ` Matthew Wilcox
2009-03-05 17:15 ` Matthew Wilcox
2009-03-05 23:26 ` Yinghai Lu
2009-03-05 23:45 ` Prakash Punnoor
2009-03-06 1:10 ` Matthew Wilcox
2009-03-06 4:15 ` Yinghai Lu
2009-03-06 9:10 ` Prakash Punnoor
2009-03-21 2:29 ` [PATCH] pci: don't enable too much HT MSI mapping -v5 -resend Yinghai Lu
2009-03-26 23:10 ` Jesse Barnes
2009-03-28 12:34 ` Prakash Punnoor
2009-03-28 13:31 ` Prakash Punnoor
2009-03-28 20:18 ` Yinghai Lu
2009-03-28 22:11 ` Prakash Punnoor
2009-03-28 20:52 ` Yinghai Lu
2009-03-28 22:16 ` Prakash Punnoor
2009-03-28 23:15 ` Yinghai Lu
2009-03-29 1:33 ` Yinghai Lu
2009-03-29 9:15 ` Prakash Punnoor
2009-03-29 9:32 ` Yinghai Lu
2009-03-29 12:50 ` Prakash Punnoor
2009-03-29 19:27 ` Yinghai Lu
2009-03-29 19:30 ` Yinghai Lu [this message]
2009-03-30 19:17 ` [PATCH] pci: don't enable too much HT MSI mapping -v6 Jesse Barnes
2009-02-22 23:42 ` [PATCH] pci: enable MSI on 8132 Matthew Wilcox
2009-02-22 3:42 ` Grant Grundler
2009-02-18 3:12 ` mpt fusion broken sometime since 2.6.24 david
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=49CFCC3D.5020205@kernel.org \
--to=yinghai@kernel.org \
--cc=akpm@linux-foundation.org \
--cc=david@lang.hm \
--cc=ebiederm@xmission.com \
--cc=hancockrwd@gmail.com \
--cc=jbarnes@virtuousgeek.org \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-pci@vger.kernel.org \
--cc=matthew@wil.cx \
--cc=mingo@elte.hu \
--cc=prakash@punnoor.de \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox